package config

import (
	"kpt-tmr-group/util/di"
	log "kpt-tmr-group/util/logger"
	"os"
	"strings"
	"sync"
)

const AppName = "kptEvent"

var (
	Module   = di.Provide(Options)
	options  *AppConfig
	appEnv   string
	initOnce sync.Once
)

// AppConfig store all configuration options
type AppConfig struct {
	AppName        string `yaml:"app_name"`
	AppEnv         string `yaml:"app_environment"`
	Debug          bool   `yaml:"debug" env:"APP_DEBUG"`
	HTTPServerAddr string `yaml:"http_server_addr" env:"HTTP_SERVER_ADDR"`

	// 数据库配置 额外加载文件部分 database.yaml
	StoreSetting StoreSetting `json:"storeSetting" yaml:"store"`
}

// StoreSetting 数据库配置
type StoreSetting struct {
	// 开启 SyDb SQL 记录
	ShowSQL         bool   `yaml:"show_sql" env:"STORE_SHOW_SQL"`
	KptEventDSNRW   string `yaml:"kpt_event_rw" env:"LINGO_COURSE_DSN_RW"`
	KptEventDSNMigr string `yaml:"kpt_event_migr" env:"LINGO_COURSE_DSN_MIGR"`
}

func Options() *AppConfig {
	return options
}

func init() {
	appEnv = strings.ToLower(os.Getenv("APP_ENVIRONMENT"))

	cfg := &AppConfig{}
	var err error
	initOnce.Do(func() {
		switch appEnv {
		default:
			err = Initialize("app.test.yaml", cfg)
			log.SetLevel(log.ErrorLevel)
		case "development":
			err = Initialize("app.develop.yaml", cfg)
		case "production":
			err = Initialize("app.production.yaml", cfg)
		}
		if err != nil {
			log.Fatalf("%+v", err)
		}
		options = cfg
	})
}
